@@ -18,22 +18,20 @@ from clipping.hints import (Mix,
                             Multisegment,
                             Point,
                             Segment)
+from . import bounding_box
 from .event import MixedEvent as Event
-from .events_queue import (EventsQueue,
-                           EventsQueueKey)
-from .sweep_line import SweepLine
+from .events_queue import (BinaryEventsQueue as EventsQueue,
+                           BinaryEventsQueueKey as EventsQueueKey)
+from .sweep_line import BinarySweepLine as SweepLine
 from .utils import (all_equal,
-                    are_bounding_boxes_disjoint,
-                    flatten,
+                    contour_to_segments,
                     sort_pair,
-                    to_bounding_box,
                     to_mixed_base,
                     to_multipolygon_contours,
                     to_multipolygon_x_max,
                     to_multisegment_x_max,
                     to_rational_multipolygon,
-                    to_rational_multisegment,
-                    to_segments)
+                    to_rational_multisegment)
 
 
 class Operation(ABC):
@@ -59,14 +57,8 @@ class Operation(ABC):
 
     __repr__ = generate_repr(__init__)
 
-    def are_operands_bounding_boxes_disjoint(self) -> bool:
-        return are_bounding_boxes_disjoint(
-                to_bounding_box(flatten(self.multisegment)),
-                to_bounding_box(flatten(border
-                                        for border, _ in self.multipolygon)))
-
     @abstractmethod
-    def compute(self) -> Union[Multipolygon, Mix]:
+    def compute(self) -> Union[Multisegment, Mix]:
         """
         Computes result of the operation.
         """
@@ -86,7 +78,7 @@ class Operation(ABC):
                                   else below_event.in_out)
         event.in_result = self.in_result(event)
 
-    def detect_intersection(self, below_event: Event, event: Event) -> int:
+    def detect_intersection(self, below_event: Event, event: Event) -> bool:
         below_segment, segment = below_event.segment, event.segment
         relationship = segments_relationship(below_segment, segment)
         if relationship is SegmentsRelationship.OVERLAP:
@@ -153,7 +145,7 @@ class Operation(ABC):
         for segment in self.multisegment:
             self.register_segment(segment, True)
         for contour in to_multipolygon_contours(self.multipolygon):
-            for segment in to_segments(contour):
+            for segment in contour_to_segments(contour):
                 self.register_segment(segment, False)
 
     @abstractmethod
@@ -169,12 +161,8 @@ class Operation(ABC):
                 to_rational_multisegment(multisegment),
                 to_rational_multipolygon(multipolygon))
 
-    def process_event(self, event: Event, processed_events: List[Event],
-                      sweep_line: SweepLine) -> None:
-        start_x, _ = event.start
-        sweep_line.move_to(start_x)
+    def process_event(self, event: Event, sweep_line: SweepLine) -> None:
         if event.is_right_endpoint:
-            processed_events.append(event)
             event = event.complement
             if event in sweep_line:
                 above_event, below_event = (sweep_line.above(event),
@@ -183,7 +171,6 @@ class Operation(ABC):
                 if above_event is not None and below_event is not None:
                     self.detect_intersection(below_event, above_event)
         elif event not in sweep_line:
-            processed_events.append(event)
             sweep_line.add(event)
             above_event, below_event = (sweep_line.above(event),
                                         sweep_line.below(event))
@@ -217,9 +204,17 @@ class Difference(Operation):
     __slots__ = ()
 
     def compute(self) -> Multisegment:
-        if (not (self.multisegment and self.multipolygon)
-                or self.are_operands_bounding_boxes_disjoint()):
-            # at least one of the arguments is empty
+        if not (self.multisegment and self.multipolygon):
+            return self.multisegment
+        multisegment_bounding_box = (bounding_box
+                                     .from_multisegment(self.multisegment))
+        if bounding_box.disjoint_with(
+                multisegment_bounding_box,
+                bounding_box.from_multipolygon(self.multipolygon)):
+            return self.multisegment
+        self.multipolygon = bounding_box.to_coupled_polygons(
+                multisegment_bounding_box, self.multipolygon)
+        if not self.multipolygon:
             return self.multisegment
         self.normalize_operands()
         return [event.segment for event in self.sweep() if event.in_result]
@@ -237,52 +232,36 @@ class Difference(Operation):
             start_x, _ = event.start
             if start_x > left_x_max:
                 break
-            self.process_event(event, result, sweep_line)
+            self.process_event(event, sweep_line)
+            if event.is_right_endpoint:
+                result.append(event.complement)
         return result
 
 
 class Intersection(Operation):
     __slots__ = ()
 
-    def compute(self) -> Mix:
-        if (not (self.multisegment and self.multipolygon)
-                or self.are_operands_bounding_boxes_disjoint()):
-            return [], [], []
+    def compute(self) -> Multisegment:
+        if not (self.multisegment and self.multipolygon):
+            return []
+        multisegment_bounding_box = (bounding_box
+                                     .from_multisegment(self.multisegment))
+        multipolygon_bounding_box = (bounding_box
+                                     .from_multipolygon(self.multipolygon))
+        if bounding_box.disjoint_with(multisegment_bounding_box,
+                                      multipolygon_bounding_box):
+            return []
+        self.multisegment = bounding_box.to_intersecting_segments(
+                multipolygon_bounding_box, self.multisegment)
+        self.multipolygon = bounding_box.to_intersecting_polygons(
+                multisegment_bounding_box, self.multipolygon)
+        if not (self.multisegment and self.multipolygon):
+            return []
         self.normalize_operands()
-        events = sorted(self.sweep(),
-                        key=EventsQueueKey)
-        multipoint = []  # type: Multipoint
-        border_multisegment = []  # type: Multisegment
-        for start, same_start_events in groupby(events,
-                                                key=attrgetter('start')):
-            same_start_events = list(same_start_events)
-            if (any(event.from_left and not (event.in_result
-                                             or event.is_right_endpoint
-                                             and event.complement.in_result)
-                    for event in same_start_events)
-                    and not all_equal(event.from_left
-                                      for event in same_start_events)):
-                no_segment_found = True
-                for event, next_event in zip(same_start_events,
-                                             same_start_events[1:]):
-                    if (event.from_left is not next_event.from_left
-                            and event.segment == next_event.segment):
-                        no_segment_found = False
-                        if not event.is_right_endpoint:
-                            border_multisegment.append(next_event.segment)
-                if (no_segment_found
-                        and all(not (event.in_result
-                                     or event.is_right_endpoint
-                                     and event.complement.in_result)
-                                for event in same_start_events)):
-                    multipoint.append(start)
-        inside_multisegment = [event.segment
-                               for event in events
-                               if event.in_result]
-        return multipoint, border_multisegment + inside_multisegment, []
+        return [event.segment for event in self.sweep() if event.in_result]
 
     def in_result(self, event: Event) -> bool:
-        return event.from_left and not event.other_in_out
+        return event.from_left and (not event.other_in_out or event.overlaps)
 
     def sweep(self) -> List[Event]:
         self.fill_queue()
@@ -295,5 +274,64 @@ class Intersection(Operation):
             start_x, _ = event.start
             if start_x > min_max_x:
                 break
-            self.process_event(event, result, sweep_line)
+            self.process_event(event, sweep_line)
+            if not event.is_right_endpoint:
+                result.append(event)
+        return result
+
+
+class CompleteIntersection(Operation):
+    __slots__ = ()
+
+    def compute(self) -> Mix:
+        if not (self.multisegment and self.multipolygon):
+            return [], [], []
+        multisegment_bounding_box = (bounding_box
+                                     .from_multisegment(self.multisegment))
+        multipolygon_bounding_box = (bounding_box
+                                     .from_multipolygon(self.multipolygon))
+        if bounding_box.disjoint_with(multisegment_bounding_box,
+                                      multipolygon_bounding_box):
+            return [], [], []
+        self.multisegment = bounding_box.to_intersecting_segments(
+                multipolygon_bounding_box, self.multisegment)
+        self.multipolygon = bounding_box.to_intersecting_polygons(
+                multisegment_bounding_box, self.multipolygon)
+        if not (self.multisegment and self.multipolygon):
+            return [], [], []
+        self.normalize_operands()
+        events = sorted(self.sweep(),
+                        key=EventsQueueKey)
+        multipoint = []  # type: Multipoint
+        border_multisegment = []  # type: Multisegment
+        for start, same_start_events in groupby(events,
+                                                key=attrgetter('start')):
+            same_start_events = list(same_start_events)
+            if (all(not (event.in_result or event.is_right_endpoint
+                         and event.complement.in_result)
+                    for event in same_start_events)
+                    and not all_equal(event.from_left
+                                      for event in same_start_events)):
+                multipoint.append(start)
+        inside_multisegment = [event.segment
+                               for event in events
+                               if event.in_result]
+        return multipoint, border_multisegment + inside_multisegment, []
+
+    def in_result(self, event: Event) -> bool:
+        return event.from_left and (not event.other_in_out or event.overlaps)
+
+    def sweep(self) -> List[Event]:
+        self.fill_queue()
+        result = []
+        sweep_line = SweepLine()
+        min_max_x = min(to_multisegment_x_max(self.multisegment),
+                        to_multipolygon_x_max(self.multipolygon))
+        while self._events_queue:
+            event = self._events_queue.pop()
+            start_x, _ = event.start
+            if start_x > min_max_x:
+                break
+            self.process_event(event, sweep_line)
+            result.append(event)
         return result
